﻿# -*- coding: utf-8 -*-

### Python scripts for studying possible tipping interactions between the Greenland Ice Sheet (GIS) and Atlantic Meridional Overturning Circulation (AMOC)
# These python scripts are relevant for the analysis and figures presented in the following publication: 
# Klose A K, Donges J F, Feudel U, Winkelmann R, 2024, Rate-induced tipping cascades arising from interactions between the Greenland Ice Sheet and the Atlantic Meridional Overturning Circulation, Earth System Dynamics 

###############################################################################
# GIS_Wood_sim_sc_C.py 
############################################################################### 

##############################################################################################################################################################

# numerical simulation of coupled GIS-AMOC system (with GIS represented by ice thickness equation with melt-elevation feedback and AMOC given by global ocean box model) 
# under constant-in-time AMOC hosing 
# with linearly decreasing GIS surface mass balance on the ground level, 
# with varying ramping rates and final values of GIS surface mass balance on the ground level 

# - to study GIS->AMOC dynamics under different GIS deglaciation pathways (unidirectional coupling; Sect. 4.2) 
# - to study GIS<-->AMOC dynamics taking into account potential negative, stabilizing feedback (bidirectional coupling; Sect. 4.3) 

# see README for parameter settings for creating figures in main manuscript 

##############################################################################################################################################################

#### import 
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as colors
from matplotlib.collections import LineCollection
import scipy.integrate as integrate
from scipy.integrate import simps
from cycler import cycler
from matplotlib.lines import Line2D
from mpl_toolkits.axes_grid1.inset_locator import inset_axes,mark_inset

######################################################
# parameters for evolution of tipping element drivers
# * GIS: surface mass balance on the ground level 
# * AMOC: hosing  
######################################################

#### CO2 concentration conditions 
# (note: only 1CO2 has been considered)

C_lev = '1CO2'

#### start value for GIS surface mass balance on the ground level
print("### Setting initial GIS surface mass balance on the ground level for simulation") 

a_0s = -0.3               

#### maximum value of GIS surface mass balance on the ground level (beyond GIS deglaciation threshold), 
#### determined as a_i - del_ai
print("### Setting final GIS surface mass balance on the ground level") 

## (1) GIS deglaciation threshold                        
a_i = -0.4331       
# -0.5 (for unidirectional coupling; for all scenarios decrease of GIS surface mass balance on ground level beyond GIS deglaciation threshold) 
# -0.4331 (for bidirectional coupling) 
## (2) distance beyond GIS deglaciation threshold a_i 
del_ai = 3.05 
# 2.5 (for overshoot / bifurcation cascade, Figure 2c in main manuscript) 
# 3.05 (for rate-induced cascade, Figure 2d in main manuscript) 

#### AMOC hosing 
print("### Setting constant AMOC hosing") 

# start value for AMOC hosing 
H_0 = 0.16*10**6      
# end value for AMOC hosing     
H_end = 0.16*10**6         

# (note: only scenarios with H_0 = H_end have been considered)

#### ramping rates of tipping element drivers 
print("### Setting ramping rate of GIS surface mass balance on the ground level") 

# ramping rate for GIS surface mass balance on the ground level
# (note: this has to be adjusted as well below when changed here) 
ra = -0.001  
# ramping rate for AMOC hosing 
# (note: this should not be changed, only scenarios with constant AMOC hosing have been considered)           
rH = 0                 

##########################################
# parameters for GIS-AMOC coupling 
##########################################

print("### Setting GIS-AMOC coupling parameters") 

#### coupling: GIS --> AMOC          
gt = 1    

#### coupling: GIS <-- AMOC
tg = 1
# scenario for evolution of GIS surface mass balance on the ground level, if tg = 1:
# 'A0.5' (strong decrease of GIS surface mass balance on the ground level)
# 'B' (limited decrease of GIS surface mass balance on the ground level)
szen = 'B' 
# coupling strength: GIS <-- AMOC                
con_oa = 2.857           
# (default: 1/0.35 = 2.857)  

#############################################
# time stepping 
#############################################

dt = 0.1        

###########################################################################
###########################################################################

# conversion between years and second
Y = 3.15e7           

###########################################################################
# parameters for GIS one-dimensional ice sheet model including melt-elevation feedback
###########################################################################

print("### Setting GIS model parameters: grid properties and physical constants") 

#### setup of GIS model grid 
m = 241                   # number of grid points   
L = 1.2e6                 # length of domain (in m) 
x = np.linspace(0,L,m)    # horizontal distance
dx = L/(m-1)              # spacing of grid (in m)
D = np.zeros(m)           # initialize array for diffusion coefficients

#### physical constants
# following Greve: Dynamics of ice sheet and glaciers  
A = 1e-16/((Y**2)**3)     # ice flow parameter (in a^-1 Pa^-3)
rho = 916.7               # ice density (in kg m^-3)
grav = 9.81*(Y**2)        # gravitation constant  (in m s^-2)
n = 3                     # exponent of Glen's flow law 
# following Levermann and Winkelmann (2016)
gam = 0.044               # melting sensitivity  (in m a^-1 °C^-1)
zet = 0.005               # atmospheric lapse rate (in °C m^-1)

con_iw = 0.9167           # conversion constant: conversion from ice volume to water volume

w_GIS = 1000000           # ice sheet length (in m) (to approximately determine GIS volume) 

###########################################################################
# parameters for AMOC box model of the global ocean
###########################################################################

print("### Setting AMOC model parameters") 

## following Alkhayuon et al. (2019); Wood et al. (2019)

#### 1xCO2
if C_lev == '1CO2': 

    # box volume (in m^3)
    V_n = 0.3261*10**17    
    V_t = 0.7777*10**17
    V_s = 0.8897*10**17
    V_ip = 2.2020*10**17
    V_b = 8.6490*10**17
    
    # temperatures  
    T_s = 4.773  #  (in °C)
    T_0 = 2.65   #  (in °C)

    # parameters determining AMOC overturning strength    
    mu =  5.5*10**(-8)  # (in °Cm^-3s)
    l = 2.79*10**7      # (in m^6kg^-1s^-1)
  
    #   
    K_n =  5.456*10**6  # (in m^3s^-1)
    K_s = 5.447*10**6   # (in m^3s^-1)
    K_ip = 96.817*10**6 # (in m^3s^-1)
    eta = 74.492*10**6  # (in m^3s^-1)
    gamma = 0.39 # 0.58 # 0.39 (no dimension)      
                    
    # constants to determine hosing  
    A_n = 0.070
    A_t = 0.752
    A_s = -0.257
    A_ip = -0.565
    
    F_n = 0.384
    F_s = 1.078
    F_t = -0.723
    F_ip = -0.739


#### 2xCO2 (not used here)
if C_lev == '2CO2': 

    # box volume (in m^3)
    V_n = 0.3683*10**17    
    V_t = 0.5418*10**17
    V_s = 0.6097*10**17
    V_ip = 1.486*10**17
    V_b = 9.925*10**17
    
    # temperatures  
    T_s = 7.919  # (in °C)
    T_0 = 3.87   # (in °C)
    
    # parameters determining overturning strength 
    mu =  22*10**(-8)  # (in °Cm^-3s)
    l = 1.62*10**7     # (in m^6kg^-1s^-1)
    
    # 
    K_n =  1.762*10**6  # (in m^3s^-1)
    K_s = 1.872*10**6   # (in m^3s^-1)
    K_ip = 99.977*10**6 # (in m^3s^-1)
    eta = 33.264*10**6  # (in m^3s^-1)
    gamma = 0.36        # (no dimension)
    
    # constants to determine hosing  
    A_n = 0.131
    A_t = 0.696
    A_s = -0.263
    A_ip = -0.564
    
    F_n = 0.486
    F_s = 1.265
    F_t = -0.997
    F_ip = -0.754


#### reference salinity
S0 = 0.035           

#### thermal and haline coefficients
alpha = 0.12
beta = 790.0

#### determine total salinity content
# from initial values of five box model 
S_n = (0.034912 -0.035)*100
S_t = (0.035435-0.035)*100
S_s = (0.034427-0.035)*100
S_ip = (0.034668-0.035)*100
S_b = (0.034538-0.035)*100

C = (V_n*S_n+V_t*S_t+V_s*S_s+V_ip*S_ip+V_b*S_b)/100 + S0*(V_n+V_t+V_s+V_ip+V_b)


###############################################################################
###############################################################################

# loading of some pre-determined data 
# * AMOC fixed points for constant AMOC hosing H_0 (used as initial values) 
# * AMOC fixed points depending on GIS freswater flux F_GIS
# * GIS freshwater flux threshold depening on AMOC hosing H 

#################################################################

#### AMOC fixed points depending on hosing H
#### this data has been determined and saved using XPPAUT 
print("### Read in data: AMOC long-term behavior depending on hosing H")  

#### load data 
X_bif = np.loadtxt('./Results/Klose_et_al_rev/AMOC/Wood_eq_sc_%s_more_C.dat'%C_lev)

#### find AMOC states for constant hosing H = H_0 of experiment (to be used as initial state for the AMOC) 

## (1) AMOC 'on' state 
pos = np.where(X_bif[:,0] == H_0)[0][0]
# box salinities 
S_N0 = X_bif[pos,1]
S_T0 = X_bif[pos,2] 
S_S0 = X_bif[pos,3]  
S_IP0 = X_bif[pos,4] 
# overturning strength 
q = (l*(alpha*(T_s-T_0)+beta/1000*((35+S_N0*10)-(35+S_S0*10))))/(1+l*alpha*mu) 
print('The initial AMOC overturning strength for this experiment is: q_start = %s'%np.round(q/(10**6),5))
# temperature of North Atlantic box 
T_N0 = mu*q + T_0

## (2) AMOC 'off' state 
pos_e = np.where(X_bif[:,0] == H_0)[0][-1]
# box salinities 
S_Nend = X_bif[pos_e,1]
S_Tend = X_bif[pos_e,2]
S_Send = X_bif[pos_e,3]  
# overturning strength 
q_end = (l*(alpha*(T_s-T_0)+beta/1000*((35+S_Nend*10)-(35+S_Send*10))))/(1+l*alpha*mu)
# temperature of North Atlantic box 
T_Nend = mu*q_end + T_0

## (3) AMOC reference state (for hosing H = 0) 
# corresponding temperature of the North Atlantic box is required for coupling: GIS <-- AMOC 
H_ref = 0
pos_ref = np.where(X_bif[:,0] == H_ref)[0][0]
# box salinities
S_Nref = X_bif[pos_ref,1]
S_Sref = X_bif[pos_ref,3] 
# overturning strength  
q_ref = (l*(alpha*(T_s-T_0)+beta/1000*((35+S_Nref*10)-(35+S_Sref*10))))/(1+l*alpha*mu)
# temperature of North Atlantix box 
T_Nref = mu*q_ref + T_0

###############################################################################

#### AMOC fixed points depending on GIS freshwater flux F_GIS
#### this data has been determined and saved using XPPAUT

print("### Read in data: AMOC long-term behavior depending on imposed GIS freshwater flux F_GIS") 

#### load for initialization of arrays 
data = np.loadtxt('./Results/Klose_et_al_rev/AMOC/bif_diagram_SN_%s_H_%s_per.dat'%(C_lev,H_0/(1e6)))
branch = data[:,4] ==1
X_bif_F = np.zeros((sum(branch),5))  
X_bif_F[:,0] = np.array(data[branch,0])

X_bif_F2 = np.zeros((len(data),5))  
X_bif_F2 [:,0] = np.array(data[:,0])

#### load data from different boxes 
var = np.array(['SN','ST','SS','SIP'])
for i in range(len(var)):
    data_read = np.loadtxt('./Results/Klose_et_al_rev/AMOC/bif_diagram_%s_%s_H_%s_per.dat'%(var[i],C_lev,H_0/(1e6)))
    X_bif_F[:,i+1] = data_read[branch,1]
    X_bif_F2[:,i+1] = data_read[:,2] 
                   
#### determine oveturning strength q                            
q_F = (l*(alpha*(T_s-T_0)+beta/1000*((X_bif_F[:,1]*1000)-(X_bif_F[:,3]*1000))))/(1+l*alpha*mu)
q_F2 = (l*(alpha*(T_s-T_0)+beta/1000*((X_bif_F2[:,1]*1000)-(X_bif_F2[:,3]*1000))))/(1+l*alpha*mu)

#### find stable and unstable branches
stable_on = np.logical_and(q_F>0,data[branch,3] == 1)
stable_off = np.logical_and(q_F<0,data[branch,3] == 1)
F_GIS_uns = np.logical_and(q_F>0,data[branch,0] >=0.1278719)

###############################################################################

#### GIS freshwater flux threshold depending on AMOC hosing H
#### this data has been determined and saved using XPPAUT

print("### Read in data: GIS freshwater flux threshold depending on AMOC hosing H") 

#### load data  
X_bifF = np.loadtxt('./Results/Klose_et_al_rev/AMOC/FGIS_crit_H_%s.dat'%C_lev) 
 
# determine GIS freshwater flux                                                 
FGIScrit = X_bifF[np.where(X_bifF[:,1] <= H_0/10**6)[0][-1],0]
                      
###############################################################################
###############################################################################

###############################################################################
# definition of functions 
###############################################################################

print("### Definition of functions / model") 

##### (1) Vialov profile 
def vialov(L,m):
    
    L_half  = L/2.                             # length of half-domain (in m)
    x_vialov = np.linspace(-L_half,L_half,m)   # horizontal distance from ice divide (m)
    h_vialov = np.zeros(m)
    
    A0 = 2./(n+2.)*A*(rho*grav)**n
    h0 = 2**(n/(2.*n+2.))*(a/A0)**(1./(2.*n+2))*L_half**0.5   # maximum surface elevation at ice divide
    
    for j in np.arange(2,m-2,1):
        if x_vialov[j]<=0:
            h_vialov[j] = h0*(1-(np.abs(x_vialov[j])/L_half)**((n+1.)/n))**(n/(2.*n+2))
        elif x_vialov[j]>0:
            h_vialov[j] = h0*(1-(x_vialov[j]/L_half)**((n+1.)/n))**(n/(2.*n+2))
    
    return h_vialov


##### (2) explicit solution of ice thickness evolution with melt-elevation feedback 
def explicit_mefb(b,h,T_N,dx,dt):
       
    ## determine value for (time-dependent) GIS surface mass balance on the ground level, using ramping rate ra
    if ra < 0: 
            a_sim = np.max([a_0s+ra*t,a_end])
    elif ra > 0: 
            a_sim = np.min([a_0s+ra*t,a_end])
    else: 
            a_sim = a_end
            
    ## slope on staggered grid
    slope_stag = np.diff(b+h)/dx 

    j = np.arange(1,m-1,1)
    k = np.arange(1+1,m,1)

    ## ice thickness on staggered grid 
    h_stag = (h[k]+h[j])/2. 
    ## diffusivity on staggered grid 
    D[j] = 2./(n+2.)*A*h_stag**2*slope_stag[j]**(n-1)*(rho*grav*h_stag)**n
    
    ## initialize arrays for current time step 
    h_evol = np.zeros(m)
    dh_evol = np.zeros(m)

    j = np.arange(2,m-2,1)   
    k = np.arange(1,m-3,1) 

    ## determine new ice thickness 
    h_evol[j] = h[j]+(D[j]*slope_stag[j]-D[k]*slope_stag[k])*dt/dx+(a_sim+tg*gam*con_oa*(T_Nref-T_N))*dt+gam*zet*dt*h[j]
    ## determine ice thickness change of current time step  
    dh_evol[j] = (D[j]*slope_stag[j]-D[k]*slope_stag[k])/dx+(a_sim+tg*gam*con_oa*(T_Nref-T_N))+gam*zet*h[j]
    
    h = np.array(h_evol, copy=True)
    dh = np.array(dh_evol, copy = True)
    
    return h, dh

    
##### (3) AMOC box model with constant C
def fun_const(x,t): 

    ## constant AMOC hosing value H 
    H_sim = H_end
     
    ## AMOC overturning strength    
    q = (l*(alpha*(T_s-T_0)+beta*(x[0]-x[2])/100))/(1+l*alpha*mu)

    ## salinity of B-box     
    # S_bsim = (C-(V_n*x[0]+V_t*x[1]+V_s*x[2]+V_ip*x[3])/100-S0*(V_n+V_t+V_s+V_ip+V_b))/V_b
    S_bsim = (100*C-(V_n*x[0]+V_t*x[1]+V_s*x[2]+V_ip*x[3])-100*S0*(V_n+V_t+V_s+V_ip+V_b))/V_b

    ## evolution of box salinities 
    if q >= 0:
        dS_n = (q*(x[1]-x[0]) + K_n*(x[1]-x[0]) - (F_n*(10**6)+A_n*(H_sim)+np.absolute(np.min([0,del_Vmean]))/Y)*S0*100)*Y/V_n
        dS_t = (q*(gamma*x[2] + (1-gamma)*x[3] - x[1]) + K_s*(x[2]-x[1]) + K_n*(x[0]-x[1]) -(F_t*(10**6)+A_t*(H_sim))*S0*100)*Y/V_t
        dS_s = (gamma*q*(S_bsim-x[2]) + K_ip*(x[3]-x[2]) + K_s*(x[1]-x[2]) + eta*(S_bsim-x[2]) -(F_s*(10**6)+A_s*(H_sim))*S0*100)*Y/V_s
        dS_ip = ((1-gamma)*q*(S_bsim-x[3]) + K_ip*(x[2]-x[3]) - (F_ip*(10**6)+A_ip*(H_sim))*S0*100)*Y /V_ip
       
    if q < 0: 
        dS_n = (np.absolute(q)*(S_bsim-x[0]) + K_n*(x[1]-x[0]) - (F_n*(10**6)+A_n*(H_sim)+np.absolute(np.min([0,del_Vmean]))/Y)*S0*100)*Y /V_n
        dS_t = (np.absolute(q)*(x[0]-x[1]) + K_s*(x[2]-x[1]) + K_n*(x[0]-x[1]) -(F_t*(10**6)+A_t*(H_sim))*S0*100)*Y /V_t
        dS_s = (gamma*np.absolute(q)*(x[1]-x[2]) + K_ip*(x[3]-x[2]) + K_s*(x[1]-x[2]) + eta*(S_bsim-x[2]) -(F_s*(10**6)+A_s*(H_sim))*S0*100)*Y/V_s
        dS_ip = ((1-gamma)*np.absolute(q)*(x[1]-x[3]) + K_ip*(x[2]-x[3]) - (F_ip*(10**6)+A_ip*(H_sim))*S0*100)*Y /V_ip
       
    return np.array([dS_n,dS_t,dS_s,dS_ip])


  
###############################################################################
# determine initial ice thickness for initial GIS surface mass balance on the ground level 
# starting from Vialov profile for a = 0.3 
###############################################################################
    
print("### Determine initial ice thickness for simulation, starting from Vialov profile") 

#### surface mass balance used to determine Vialov profile
a = 0.3
#### initial thickness via Vialov profile for initial surface mass balance
h = vialov(L,m) 
#### bed topography b = 0               
b = np.zeros(m) 
#### ramping rate of GIS surface mass balance on the ground level
# (this is set to zero here to find stable steady GIS state given starting value of GIS surface mass balance on the ground level, starting from Vialov profile for a = 0.3) 
ra = 0
#### time settings for determining initial state 
count_t = -1
time_end_init = 60000.             

#### initial GIS surface mass balance on ground level, which is kept constant for determining initial ice thickness 
a_end = a_0s

#### loop over time 
for t in np.arange(0,time_end_init+dt,dt):
    count_t = count_t + 1
    
    ## integration of ice thickness equation, starting from Vialov profile 
    h,dh = explicit_mefb(b,h,T_N0,dx,dt)   
    
    dh[h < 0] = 0           # as a consequence of setting negative ice thickness to zero: 
                            # the rate of change must be set to zero as well                              
    h[h < 0] = 0            # enforce nonnegative ice thickness 
    
#### save final ice thickness as initial conditon
h_init = np.array(h)    
dh_init = np.array(dh)

###############################################################################
# simulation 
###############################################################################

#### ramping rate for GIS surface mass balance on the ground level
# !!!! IMPORTANT: need to be adjusted!!!!
ra = -0.001  

#### determine final GIS surface mass balance on the ground level
## unidirectional coupling
if tg == 0:          
    a_end = a_i-del_ai 
## bidirectional coupling (two different scenarios considered here, depending on effective GIS deglaciation thresholds)
if tg == 1:   
    # a_end crosses only 'first TP', stays below 'second TP' (limited decrease of GIS surface mass balance on ground level)       
    if szen == 'B':
        a_end = a_i - gam*con_oa*(T_Nref-T_Nend) + 0.001 
    # a_end crossed 'second TP' (strong decrease of GIS surface mass balance on ground level)  
    if szen == 'A0.5':
        a_end = a_i - gam*con_oa*(T_Nref-T_Nend) - 0.5 

#### determine simulation time 
time_end = (a_end - a_0s)/ra + 25000 
t_vec = np.arange(0,time_end+dt,dt)  

#### initialization of result arrays 
## ice thickness  
h_erg = np.zeros([len(np.arange(0,time_end+dt,dt)),m])  
## change of ice thickness 
dh_erg = np.zeros([len(np.arange(0,time_end+dt,dt)),m])  
## evolution of GIS surface mass balance 
a_t = np.zeros(len(np.arange(0,time_end+dt,dt)))  
a_e = np.zeros(len(np.arange(0,time_end+dt,dt)))  
## AMOC box salinities 
X_erg = np.zeros([len(np.arange(0,time_end+dt,dt)),4])  
## evolution of AMOC hosing  
H_t = np.zeros(len(np.arange(0,time_end+dt,dt)))  

#### initialize model
## initial ice thickness (as obtained from equilibrium ice thickness simulation, after starting from Vialov profile)
h = h_init                                      
## initial ice thickness change / GIS freshwater flux 
del_Vmean = con_iw*w_GIS*simps(dh_init,x)      
## bedrock elevation b = 0 
b = np.zeros(m)                                  
## AMOC box salinities 
st = np.array([S_N0,S_T0,S_S0,S_IP0])            


###########################
# time stepping
###########################

print("#### time-stepping") 

count_t = 0 

#### loop over time 
for t in np.arange(0,time_end,dt):   
    
    count_t = count_t + 1
    
    ## calculate temperature of North Atlantic box, based on AMOC overturning strength  
    q = (l*(alpha*(T_s-T_0)+beta/1000*((35+st[0]*10)-(35+st[2]*10))))/(1+l*alpha*mu)
    T_Nsim = mu*q + T_0
    
    ##### ice sheet evolution 
    h,dh = explicit_mefb(b,h,T_Nsim, dx,dt)
      
    dh[h < 0] = 0           # as a consequence of setting negative ice thickness to zero: 
                            # the rate of change must be set to zero as well 
                            
    h[h < 0] = 0            # enforce nonnegative ice thickness 
            
    ##### salinity evolution 
    ## AMOC integration via odeint
    #X = integrate.odeint(fun_const, st, np.linspace(0,dt,2), full_output=True) 
    ## AMOC intergration by hand 
    k_1 = fun_const(st,dt)
    k_2 = fun_const(st + dt/2*k_1, t + dt/2)
    k_3 = fun_const(st + dt/2*k_2, t + dt/2)
    k_4 = fun_const(st + dt*k_3, t+dt)
    ####X2 = st + dt/6 *(k_1+2*k_2+3*k_3+k_4)
    X2= st + dt/2 *(fun_const(st,t) + fun_const(st + dt*fun_const(st,t),t+count_t*dt))   
    
    #### initialization for next time step 
    #st = X[0][-1,:]                     # (box salinities from integration via odeint)
    st = X2                              # (box salinities from integration by hand)
    del_Vmean = con_iw*w_GIS*simps(dh,x)                    
        
    #### saving for current time step 
    # ice thickness 
    h_erg[count_t,:] = np.array(h)
    # ice thickness change 
    dh_erg[count_t,:] = np.array(dh)
    # box salinities 
    #X_erg[count_t,:] = np.array(X[0][-1,:])  # (box salinities from integration via odeint)
    X_erg[count_t,:] = np.array(X2)           # (box salinities from integration by hand) 
    
    #### determine GIS surface mass balance on ground level for current time step 
    if ra < 0: 
        a_t[count_t] = np.maximum(a_0s+ra*t,a_end)  
    elif ra > 0: 
        a_t[count_t] = np.minimum(a_0s+ra*t,a_end)    
    a_e[count_t] = a_t[count_t]+gam*con_oa*(T_Nref-T_Nsim)
    
    #### determine AMOC hosing for current time step (note: this does not change over time)
    if rH < 0: 
        H_t[count_t] = np.maximum(H_0+rH*t,H_end)
    elif rH > 0: 
        H_t[count_t] = np.minimum(H_0+rH*t,H_end)
    else: 
        H_t[count_t] = H_end
   
########################
# end of time stepping
########################

#### add initial conditions to result array      
X_erg[0,:] = [S_N0,S_T0,S_S0,S_IP0]
h_erg[0,:] = h_init
dh_erg[0,:] = dh_init
H_t[0] = H_0
a_t[0] = a_0s
a_e[0] = a_0s+gam*con_oa*(T_Nref-T_N0)
        
#### determine evolution of AMOC overturning strength from box salinities 
q = (l*(alpha*(T_s-T_0)+beta/1000*((35+X_erg[:,0]*10)-(35+X_erg[:,2]*10))))/(1+l*alpha*mu)

#### calculate time of GIS deglaciation 
if len(np.where(np.mean(h_erg,1) == 0)[0]) > 0: 
    t_melt = t_vec[np.where(np.mean(h_erg,1) == 0)[0][0]]
else: 
    t_melt = t_vec[-1]

q_bif = (l*(alpha*(T_s-T_0)+beta/1000*((35+X_bif[:,1]*10)-(35+X_bif[:,3]*10))))/(1+l*alpha*mu)


###############################################################################
# Plotting
###############################################################################

print("### Plotting") 

###############################################################################
###############################################################################

### FIGURE: AMOC overturning strength q over time 
### with coloring indicating percentage of initial GIS ice volume
### (Figure 4 a-b, d-e in main manuscript) 

# determine steady state AMOC overturning strength for respective constant hosing 
q_start = (l*(alpha*(T_s-T_0)+beta/1000*((35+S_N0*10)-(35+S_S0*10))))/(1+l*alpha*mu)
q_end = (l*(alpha*(T_s-T_0)+beta/1000*((35+S_Nend*10)-(35+S_Send*10))))/(1+l*alpha*mu)

# approximate GIS volume from ice sheet thickness 
V = con_iw*w_GIS*simps(h_erg,x)
# determine GIS freshwater flux  
dV = np.absolute(np.minimum(0,con_iw*w_GIS*simps(dh_erg,x)))

# percentage of initial ice volume 
V_per = V/V[0]*100

points = np.array([t_vec,q/(10**6)]).T.reshape(-1,1,2)
segments = np.concatenate([points[:-1],points[1:]],axis = 1)


fig_q, ax_q = plt.subplots(1,1,figsize = (6.0,4.5))

axins_q = inset_axes(ax_q,
                    width="100%", height="100%",
                    #bbox_to_anchor=(0.3,0.25, 0.2, 0.5), # Figure 4a and Figure 4d
                    #bbox_to_anchor=(0.3,0.25, 0.35, 0.3), # Figure 4b
                    bbox_to_anchor=(0.05,0.25, 0.55, 0.3),  # Figure 4e
                    bbox_transform=ax_q.transAxes) 

# steady state AMOC overturning strength for constant hosing 
ax_q.plot(t_vec,q_start/(10**6)+np.zeros(len(t_vec)), color = 'black',linewidth = 3)
ax_q.plot(t_vec,q_end/(10**6)+np.zeros(len(t_vec)), color = 'black',linewidth = 3)

# axis limits 
ax_q.set_xlim([t_vec[0],t_melt + len(t_vec)/500])
ax_q.set_xlim([t_vec[0],t_vec[-1]])
ax_q.set_ylim([-12.8,15])

#axins_q.set_xlim([0,5000]) # Figure 4a and Figure 4d
#axins_q.set_xlim([0,7500]) # Figure 4b
axins_q.set_xlim([0,16000]) # Figure 4e

#axins_q.set_ylim([-11,10]) # Figure 4a and Figure 4d
#axins_q.set_ylim([8.7,12]) # Figure 4b
axins_q.set_ylim([10.3,11.3]) # Figure 4e

# colormap 
cmap = plt.get_cmap('PuRd')
cmap_get = cmap(np.linspace(0, 1, 256))
pink = np.array([169/256, 169/256, 169/256, 1])
cmap_get[:50, :] = pink
cmap_new = colors.ListedColormap(cmap_get)

# AMOC overturning strength over time 
norm = colors.BoundaryNorm(boundaries=np.arange(0,110,10), ncolors=256)
lc = LineCollection(segments,cmap = cmap_new,norm = norm, zorder = 3)
lc.set_array(V_per)
lc.set_linewidth(6)
line = ax_q.add_collection(lc)

lc_zoom = LineCollection(segments,cmap = cmap_new,norm = norm, zorder = 3)
lc_zoom.set_array(V_per)
lc_zoom.set_linewidth(6)
line_zoom = axins_q.add_collection(lc_zoom)


# labeling 
title_s = 'GIS deglaciation and AMOC recovery'    # GIS stabilization; AMOC recovery
ax_q.text(ax_q.get_xlim()[0]+(ax_q.get_xlim()[1]-ax_q.get_xlim()[0])/4,q_start/(10**6)+1.5,r'strong AMOC ($F_{GIS}$ = 0 Sv)'%np.round(H_0/10**6,3), fontsize = 14.0,zorder = 1)  # 500
ax_q.text(ax_q.get_xlim()[0]+(ax_q.get_xlim()[1]-ax_q.get_xlim()[0])/4,q_end/(10**6)-2.7,r'collapsed AMOC', fontsize = 14.0,zorder = 1) # 7.5, fontsize = 19
ax_q.text(ax_q.get_xlim()[0]-(ax_q.get_xlim()[1]-ax_q.get_xlim()[0])/6.0,ax_q.get_ylim()[1]+(ax_q.get_ylim()[1]-ax_q.get_ylim()[0])/17, '(e)', fontsize = 14)
ax_q.text(ax_q.get_xlim()[0],ax_q.get_ylim()[1]+(ax_q.get_ylim()[1]-ax_q.get_ylim()[0])/17, title_s, fontsize = 14, weight = 'bold')

ax_q.tick_params(axis='both', which='major', labelsize=14)
ax_q.set_xlabel(r'time $t$ [yr]',fontsize = 14)
ax_q.set_ylabel(r'AMOC overturning strength' '\n' '$q$ [Sv]',fontsize = 14,labelpad = -9)
axins_q.set_xticklabels('')
axins_q.set_yticklabels('')

mark_inset(ax_q, axins_q, loc1 = 4, loc2 = 2, fc = "none", ec = "0.5")

fig_q.tight_layout(rect=[0, 0, 1, 0.95]) 
fig_q.savefig("Figure_4e.pdf")

## legend 
##fig_leg = plt.figure(figsize = (6.5,5.0))
##ax_leg = fig_leg.add_subplot(1,1,1)
###cbar = fig_leg.colorbar(line,ax = ax_leg)
###cbar.ax.tick_params(labelsize=15)
###cbar.set_label('percentage of initial GIS ice volume', fontsize = 15)
##ax_leg.set_xlim([t_vec[0],t_vec[-1]])
##ax_leg.set_ylim([-12,20])
##ax_leg.text(ax_leg.get_xlim()[0],ax_leg.get_ylim()[1]+(ax_leg.get_ylim()[1]-ax_leg.get_ylim()[0])/17, 'limited SMB evolution', fontsize = 18, weight = 'bold')
##ax_leg.tick_params(axis='both', which='major', labelsize=20)
##ax_leg.set_xlabel(r'$t$ [yr]',fontsize = 20)
##ax_leg.set_ylabel(r'$q$ [Sv]',fontsize = 20)
##fig_leg.tight_layout()

###############################################################################
###############################################################################

### FIGURE: AMOC overturning strength depending on GIS freshwater flux F_GIS
### coloring follows Matplotlib example: https://matplotlib.org/3.1.1/gallery/lines_bars_and_markers/multicolored_line.html
### (Figure 2c-d) in main manuscript

points = np.array([dV/Y/(10**6),q/(10**6)]).T.reshape(-1,1,2)
segments = np.concatenate([points[:-1],points[1:]],axis = 1)

fig_q = plt.figure(figsize = (6.0,4.5))  # (6.5,5.0)
ax_q = fig_q.add_subplot(1,1,1)

# axis limits 
ax_q.set_xlim([-0.14,0.17])
ax_q.set_ylim([-12,20])

# GIS freshwater flux threshold  
ax_q.plot([X_bif_F[stable_on,0][-1],X_bif_F[stable_on,0][-1]],[ax_q.get_ylim()[0],ax_q.get_ylim()[1]],'-',color = 'green', linewidth = 3) # ,zorder = 1
ax_q.text(X_bif_F[stable_on,0][-1]+0.004,ax_q.get_ylim()[1]-(ax_q.get_ylim()[1]-ax_q.get_ylim()[0])/12,r'$F_{GIS_{Hopf}}$', fontsize = 14, color = 'green')

# steady state AMOC overturning strength depending on GIS freshwater flux 
colors_eq = "black" # "black" (Figure 2d) # "#8D99AE" (Figure 2c) 
plt.plot(X_bif_F[stable_on,0],q_F[stable_on]/10**6,'-', color = colors_eq,linewidth = 4,zorder = 2,  label = 'stable equilibria') # 
plt.plot(X_bif_F[stable_off,0],q_F[stable_off]/10**6,'-', color = colors_eq, linewidth = 4, zorder = 2, label = 'unstable equilibria') # 
plt.plot(X_bif_F[data[branch,3] == 2,0][1:],q_F[data[branch,3] == 2][1:]/10**6,'--', color = colors_eq, linewidth = 4, zorder = 2) # 

# colomap showing GIS state 
cmap = plt.get_cmap('PuRd')
cmap_get = cmap(np.linspace(0, 1, 256))
pink = np.array([169/256, 169/256, 169/256, 1])
cmap_get[:50, :] = pink
cmap_new = colors.ListedColormap(cmap_get)

# AMOC trajectory in terms of overturning strength, depending on GIS freshwater flux 
norm = colors.BoundaryNorm(boundaries=np.arange(0,110,10), ncolors=256)
lc = LineCollection(segments,cmap = cmap_new,norm = norm,alpha=1, zorder = 3, label = 'trajectory') # 
V_per = V/V[0]*100
lc.set_array(V_per)
lc.set_linewidth(4)
line = ax_q.add_collection(lc)  # plot coloured line

# labeling 
ax_q.set_xlabel("GIS freshwater flux $F_{GIS}$ [Sv]", fontsize = 14)
ax_q.set_ylabel("AMOC overturning strength" "\n" "$q$ [Sv]", fontsize = 14, labelpad = -9)
ax_q.text(ax_q.get_xlim()[0]-(ax_q.get_xlim()[1]-ax_q.get_xlim()[0])/6.0,ax_q.get_ylim()[1]+(ax_q.get_ylim()[1]-ax_q.get_ylim()[0])/17, '(d)', fontsize = 14)
ax_q.text(ax_q.get_xlim()[0],ax_q.get_ylim()[1]+(ax_q.get_ylim()[1]-ax_q.get_ylim()[0])/17, 'Rate-induced cascade', fontsize = 14, weight = 'bold')
# 'Overshoot / bifurcation cascade'
# 'Rate-induced cascade'
ax_q.tick_params(axis='both', which='major', labelsize=14)

fig_q.tight_layout(rect=[0, 0, 1, 0.95]) 
#fig_q.savefig("Figure_2d.pdf")

###############################################################################
###############################################################################

