﻿# -*- coding: utf-8 -*-

### Python scripts for studying possible tipping interactions between the Greenland Ice Sheet (GIS) and Atlantic Meridional Overturning Circulation (AMOC)
# These python scripts are relevant for the analysis and figures presented in the following publication: 
# Klose A K, Donges J F, Feudel U, Winkelmann R, 2024, Rate-induced tipping cascades arising from interactions between the Greenland Ice Sheet and the Atlantic Meridional Overturning Circulation, Earth System Dynamics 

###################################################
# Plotting_CascadeCorridor.py
###################################################

######################################################################################################################

# plotting of GIS-AMOC tipping outcomes as determined by (2) GIS_Wood_TippingCorridor_sc_C.py in a bidirectionally coupled GIS-AMOC system
# depending on AMOC hosing, 
# for specific scenario of GIS surface mass balance evolution (strong / limited decrease of GIS surface mass balance on the ground level) and fixed coupling strength via relative temperature change 

######################################################################################################################

#### import 
import numpy as np
import matplotlib.pyplot as plt
import scipy.integrate as integrate
from scipy.integrate import simps
from cycler import cycler

######################################################################################################################

######################################################
# parameters for evolution of tipping element drivers
# * GIS: surface mass balance on the ground level 
# * AMOC: hosing  
######################################################

#### CO2 concentration conditions
# (note: only 1CO2 has been considered)

C_lev = '1CO2'          

#### start value for GIS surface mass balance on the ground level
 
a_0s = -0.3              

#### maximum value of GIS surface mass balance on the ground level (beyond GIS deglaciation threshold), 
#### determined as a_i - del_ai

## (1) GIS deglaciation threshold
a_i = -0.4331           
## (2) distance beyond GIS deglaciation threshold a_i
del_ai = 0.85       

#### AMOC hosing 

# start value for AMOC hosing 
H_0_v = np.arange(0.0*10**6, 0.22*10**6,0.001*10**6 )
# end value for AMOC hosing 
H_end_v = np.arange(0.0*10**6, 0.22*10**6,0.001*10**6 )
   
# (note: only scenarios with H_0 = H_end have been considered)

#### ramping rates of tipping element drivers 

# ramping rate for GIS surface mass balance on the ground level
ra = -0.001             
# ramping rate for AMOC hosing
# (note: this should not be changed, only scenarios with constant hosing H have been considered)
rH = 0.0                 

##########################################
# parameters for GIS-AMOC coupling 
##########################################

#### coupling: GIS --> AMOC 
gt = 1      

#### coupling: GIS <-- AMOC
tg = 1               
#### scenario for evolution of GIS surface mass balance on the ground level, if tg = 1:
# 'A0.5' (strong decrease of GIS surface mass balance on the ground level)
# 'B' (limited decrease of GIS surface mass balance on the ground level)
szen = 'B'              
#### coupling strength: GIS <-- AMOC   
con_oa = 2.857    
# (default: 1/0.35 = 2.857)    
#### melting sensitivity used in coupling via relative temperature change  
gam = 0.044              # (m a^-1 °C^-1)

    
#############################################
# time stepping 
#############################################
 
dt = 0.1
 
###########################################################################
# parameters for GIS one-dimensional ice sheet model including melt-elevation feedback
###########################################################################

m = 241                   # number of grid points   
L = 1.2e6                 # length of domain (m)
dx = L/(m-1)              # grid spacing (m)

###########################################################################
# parameters for AMOC box model of the global ocean
###########################################################################

### 1xCO2
# box volume (in m^3)
V_n = 0.3261*10**17    
V_t = 0.7777*10**17
V_s = 0.8897*10**17
V_ip = 2.2020*10**17
V_b = 8.6490*10**17
    
# temperatures
T_s = 4.773 # (in °C)
T_0 = 2.65  # (in °C)
    
# parameters determining AMOC overturning strength 
mu =  5.5*10**(-8)  # (in °Cm^-3s)
l = 2.79*10**7#  (m^6kg^-1s^-1)
    
#  
K_n =  5.456*10**6 # (in m^3s^-1)
K_s = 5.447*10**6 # (in m^3s^-1)
K_ip = 96.817*10**6 # (in m^3s^-1)
eta = 74.492*10**6 # (in m^3s^-1)
gamma = 0.39 # 0.58 # 0.39 # (no dimension)
    
# constants to determine hosing
A_n = 0.070
A_t = 0.752
A_s = -0.257
A_ip = -0.565
    
F_n = 0.384
F_s = 1.078
F_t = -0.723
F_ip = -0.739

#### reference salinity
S0 = 0.035               

#### thermal and haline coefficients
alpha = 0.12
beta = 790.0

###########################################################################
###########################################################################

# conversion between years and second
Y = 3.15e7

###############################################################################
###############################################################################

# loading of some pre-determined data 
# * AMOC fixed points for constant AMOC hosing H_0 (used as initial values) 
# * GIS freshwater flux threshold depening on AMOC hosing 

#################################################################

#### AMOC fixed points depending on hosing H
#### this data has been determined and saved using XPPAUT 

#### load data 
X_bif = np.loadtxt('./Results/Klose_et_al_rev/AMOC/Wood_eq_sc_%s_more_C.dat'%C_lev)

## AMOC reference state with hosing H = 0, used for coupling: GIS <-- AMOC
H_ref = 0
pos_ref = np.where(X_bif[:,0] == H_ref)[0][0]
# box salinities
S_Nref = X_bif[pos_ref,1]
S_Sref = X_bif[pos_ref,3]  
# overturning strength
q_ref = (l*(alpha*(T_s-T_0)+beta/1000*((35+S_Nref*10)-(35+S_Sref*10))))/(1+l*alpha*mu)
# temperature of North Atlantic box
T_Nref = mu*q_ref + T_0

# AMOC overturning strength 
q_bif = (l*(alpha*(T_s-T_0)+beta/1000*((35+X_bif[:,1]*10)-(35+X_bif[:,3]*10))))/(1+l*alpha*mu)

###############################################################################

#### GIS freshwater flux threshold depending on AMOC hosing H
#### this data has been determined and saved using XPPAUT

#### load data   
X_bifF = np.loadtxt('./Results/Klose_et_al_rev/AMOC/FGIS_crit_H_%s.dat'%C_lev)

#### get GIS freshwater flux threshold                                                    
FGIS_crit = X_bifF[:,0]

#### get corresponding hosing value
H_Fvec = X_bifF[:,1]

######################################################################################################################

#### load simulation data 

#### initialization of result arrays 
erg_tip = np.zeros((len(H_0_v),2))
a_end_v = np.zeros(len(H_0_v))

#### load data
if tg == 1: 
    data = np.loadtxt('./Results/Klose_et_al_rev/GIS_AMOC/GIS_AMOC_C_CascCorr_dt_%s_dx_%s_L_%s_%s_d_%s_%s_conoa_%s_a0_%s_aend_var_szen_%s_ra_%s_H0_var_Hend_var_rH_%s_Hmin_%s_Hmax_%s.dat'%(dt,dx,L,C_lev,gt,tg,np.round(con_oa,3),a_0s,szen,ra,rH,H_0_v[0],H_0_v[-1]))
    dVmax = data[4,:]
 
#### save data in array    
erg_tip[:,0] = data[2,:]
erg_tip[:,1] = data[3,:]
a_end_v = data[1,:]

######################################################################################################################

#### Plotting 

#########################################################
# determine patterns of tipping 
#########################################################

such_a = np.logical_and(erg_tip[:,0] == 1, erg_tip[:,1] == 1)
such_b = np.logical_and(erg_tip[:,0] == 0, erg_tip[:,1] == 1)
such_c = np.logical_and(erg_tip[:,0] == 0, erg_tip[:,1] == 0)
such_d = np.logical_and(erg_tip[:,0] == 1, erg_tip[:,1] == 0)

#########################################################

#### create figure 
fig = plt.figure(figsize = (6.5,5)) # 6.5,5
ax = fig.add_subplot(1,1,1)
ax_H = ax.twinx()

###################################################################
# determine GIS and AMOC thresholds for plotting in stability map  
###################################################################

#### intrinsic GIS declaciation thresholds
a_i_a = -0.4331 # H_bif[np.where(V_bif > 0)[0][0],0]  
a_i_b = 0
T_N_vec = mu*q_bif + T_0

#### find AMOC 'on' and 'off' branches 
Tn_on = np.array(T_N_vec[np.where(q_bif > 0)[0]])
F_on = np.array(X_bif[np.where(q_bif>0)[0],0])
Tn_off =  np.array(T_N_vec[np.where(q_bif <= 0)[0]])
F_off = np.array(X_bif[np.where(q_bif<=0)[0],0])

#### determine AMOC tipping point (1)
H_crit2 = X_bif[np.where(q_bif < 0)[0][0],0]
#### determine AMOC tipping point (2) 
H_crit1 = X_bif[np.where(q_bif >= 0)[0][-1],0] # 0.478*10**6

#### determine effective GIS deglaciation thresholds 
a_eff1 = a_i_a - tg*gam*con_oa*(T_Nref-Tn_on) 
a_eff2 = a_i_a - tg*gam*con_oa*(T_Nref-Tn_off) 
a_eff3 = a_i_b + tg*gam*con_oa*(T_Nref-Tn_on) 
a_eff4 = a_i_b + tg*gam*con_oa*(T_Nref-Tn_off) 

#### define upper and lower limit of GIS surface mass balance on ground for plotting
a_lower = -0.2 
a_upper = -1.2 

#### define upper and lower limit of AMOC hosing for plotting 
H_lower = 0.0 
H_upper = 0.3 

######################################
# plotting of stability map
######################################

#### effective GIS deglaciation thresholds
ax.plot(a_eff1,F_on/(10**6), c = 'k', linewidth = 3) # ax.plot(a_eff1,F_n+A_n*F_on/(10**6), c = 'k', linewidth = 3)
ax.plot(a_eff2,F_off/(10**6), c = 'k', linewidth = 3) # ax.plot(a_eff2,F_n+A_n*F_off/(10**6), c = 'k', linewidth = 3)
ax.plot(a_eff3,F_on/(10**6), c = 'k', linewidth = 3) # ax.plot(a_eff3,F_n+A_n*F_on/(10**6), c = 'k', linewidth = 3)
ax.plot(a_eff4,F_off/(10**6), c = 'k', linewidth = 3) # ax.plot(a_eff4,F_n+A_n*F_off/(10**6), c = 'k', linewidth = 3)

#### axis settings
ax.set_xlim([a_upper,a_lower])
ax.set_ylim([H_lower,H_upper]) # ax.set_ylim([F_n + A_n*H_lower,F_n + A_n*H_upper])

#### AMOC hosing threshold
ax.plot([ax.get_xlim()[0],ax.get_xlim()[1]],[H_crit1/(10**6),H_crit1/(10**6)], c = 'k',linewidth = 3) # AMOC TP (1) 

#### instrinsic GIS deglaciation thresholds 
if tg == 1: 
    ax.plot([a_i_a,a_i_a],[ax.get_ylim()[0],ax.get_ylim()[1]],'--',c = 'gray')
    ax.plot([a_i_b,a_i_b],[ax.get_ylim()[0],ax.get_ylim()[1]],'--',c = 'gray')


#########################################
# plotting of tipping pattern
#########################################

#### (GIS: ice-free; AMOC: collapsed)
if np.sum(such_a) > 0:  
    lim_a = np.where(np.concatenate(([such_a[0]],such_a[:-1] != such_a[1:],[True])))[0]
    if len(lim_a) > 1: 
        for i in np.arange(0,len(lim_a)-1,2): 
            lim_a_vec = a_end_v[lim_a[i]:(lim_a[i+1]-1)+1]
            ax.fill_betweenx(np.linspace(H_end_v[lim_a[i]]/(10**6),H_end_v[lim_a[i+1]-1]/(10**6),len(lim_a_vec)),np.zeros(len(lim_a_vec))+a_0s,lim_a_vec,color = 'darkgray', alpha = 1.0)
    
#### (GIS: ice-covered; AMOC: collapsed)    
if np.sum(such_b) > 0:  
    lim_b = np.where(np.concatenate(([such_b[0]],such_b[:-1] != such_b[1:],[True])))[0]
    print(lim_b)
    if len(lim_b) > 1: 
        for i in np.arange(0,len(lim_b)-1,2): 
            lim_b_vec = a_end_v[lim_b[i]:(lim_b[i+1]-1)+1]
            ax.fill_betweenx(np.linspace(H_end_v[lim_b[i]]/(10**6),H_end_v[lim_b[i+1]-1]/(10**6),len(lim_b_vec)),np.zeros(len(lim_b_vec))+a_0s,lim_b_vec,color = [0.40392157, 0., 0.12156863, 1.], alpha = 1.0) 

#### (GIS: ice-covered; AMOC: strong)
if np.sum(such_c) > 0: # (nicht gekippt, nicht gekippt)
    lim_c = np.where(np.concatenate(([such_c[0]],such_c[:-1] != such_c[1:],[True])))[0]
    if len(lim_c) > 1: 
        for i in np.arange(0,len(lim_c)-1,2): 
            lim_c_vec = a_end_v[lim_c[i]:(lim_c[i+1]-1)+2]
            ax.fill_betweenx(np.linspace(H_end_v[lim_c[i]]/(10**6),H_end_v[lim_c[i+1]-1]/(10**6),len(lim_c_vec)),np.zeros(len(lim_c_vec))+a_0s,lim_c_vec,facecolor = [0.40392157, 0., 0.12156863, 1.], hatch=r"//",  edgecolor = 'white',alpha = 1.0)

#### (GIS: ice-free; AMOC: strong)
if np.sum(such_d) > 0: # (gekippt, nicht gekippt)
    lim_d = np.where(np.concatenate(([such_d[0]],such_d[:-1] != such_d[1:],[True])))[0]
    if len(lim_d) > 1: 
        for i in np.arange(0,len(lim_d)-1,2): 
            lim_d_vec = a_end_v[lim_d[i]:(lim_d[i+1]-1)+1]
            ax.fill_betweenx(np.linspace(H_end_v[lim_d[i]]/(10**6),H_end_v[lim_d[i+1]-1]/(10**6),len(lim_d_vec)),np.zeros(len(lim_d_vec))+a_0s,lim_d_vec,facecolor = 'white', hatch=r"//",  edgecolor = 'darkgray',alpha = 1.0)

#########################################################

dy = ax.get_ylim()[1]-ax.get_ylim()[0]
dx = ax.get_xlim()[1]-ax.get_xlim()[0]

#### add labeling of thresholds 
# AMOC hosing threshold
ax.text(ax.get_xlim()[0]+0.016*dx,H_crit1/(10**6)+0.02*dy,r'$H_{Hopf}$', fontsize = 14)
# GIS (de)glaciation threshold
if tg == 1: 
    ax.text(a_eff2[-1]-0.1*dx, ax.get_ylim()[1]-0.1*dy, r'$a_{0_{dgc}}^{(2)}$', fontsize = 14)
    ax.text(a_eff1[0],ax.get_ylim()[0]+0.03*dy, r'$a_{0_{dgc}}^{(1)}$', fontsize = 14)
    ax.text(ax.get_xlim()[0]-dx/7.5,ax.get_ylim()[1]+dy/55,r'(f)', fontsize = 14)
    
if tg == 0: 
    ax.text(a_eff2[-1]-0.15*dx, ax.get_ylim()[1]-0.08*dy, r'$a_{0_{dgc}}$', fontsize = 14)
    ax.text(a_eff4[-1]+0.008*dx, ax.get_ylim()[1]-0.08*dy, r'$a_{0_{gc}}$', fontsize = 14)
    ax.text(ax.get_xlim()[0]+0.01*dx,ax.get_ylim()[1]-0.08*dy,'(b)', fontsize = 14)

#### add arrows to indicate specific scenarios (compare Figure 4)
## (strong GIS surface mass balance decrease in Figure 4f) 
#ax.arrow(-0.3, 0.16, -1.06+0.3, 0.0, color = 'green',linewidth = 2,head_width=0.01, head_length=0.03) 
#ax.text(-0.3+0.01, 0.16-0.0075,r'$(b)$', fontsize = 14, color = 'green')
#ax.arrow(-0.3, 0.205, -1.065+0.3, 0, color = 'green',linewidth = 2,head_width=0.01, head_length=0.03)    
#ax.text(-0.3+0.01, 0.205-0.0075,r'$(a)$', fontsize = 14, color = 'green')

## (limited GIS surface mass balance decrease in Figure 4c)
ax.arrow(-0.3, 0.16, -0.557+0.3, 0.0, color = 'green',linewidth = 2,head_width=0.01, head_length=0.03) 
ax.text(-0.3+0.01, 0.16-0.0075,r'$(e)$', fontsize = 14, color = 'green')
ax.arrow(-0.3, 0.205, -0.56+0.3, 0, color = 'green',linewidth = 2,head_width=0.01, head_length=0.03)    
ax.text(-0.3+0.01, 0.205-0.0075,r'$(d)$', fontsize = 14, color = 'green')
    
#### axis settings 
ax.set_xlabel(r'GIS surface mass balance $a_0$ [m a$^{-1}$]', fontsize = 14)
ax.tick_params(axis='both', which='major', labelsize=14)
ax.tick_params(labelleft=False) 
ax_H.set_ylim([H_lower,H_upper])
ax_H.set_ylabel(r'AMOC hosing $H$ [Sv]', fontsize = 14)
ax_H.tick_params(axis='both', which='major', labelsize=14)

fig.tight_layout(rect=[0.05, 0, 1, 0.95])


#### saving 
fig.savefig("Figure_4c.pdf")
 